import numpy as np
import os
import sys
import pandas as pd
import matplotlib.pyplot as plt
import datetime
import seaborn as sns


sys.path.insert(1, os.getcwd() )

OUTPUT_FOLDER='\\output_figures' 
DATA_SOURCE='\\data'

dfPlot=pd.read_csv(os.getcwd() + DATA_SOURCE +"/weekly_counts.csv").drop(['Unnamed: 0'],axis=1)

dfPlot.columns.values
dfPlot['week_start_date']=pd.to_datetime(dfPlot['week_start_date'])

dfPlot.columns.values
weeksColNm='week_of_year'
groupbyNm='week_start_date_year'

"""Plot Categories"""
ageGroupArrayCols=['All','2_40 and above', '1_16-39']
AgeGroupNamingArray=['All','40 and above', '16-39']
calltypeCols=['CA','ACS']
calltypeColsUnderscore=[item.replace(" ", "_") for item in calltypeCols]
calltypeColsUnderscoreNamingArray=['Cardiac arrest','Acute coronary\nsyndrome']


for idxageOver40, ageOver40 in enumerate(ageGroupArrayCols):
    dfPlotTemp=dfPlot.copy().reset_index(drop=True)
    if ageOver40 =='All':
        print('Use All') 
    else: 
        dfPlotTemp=dfPlotTemp[dfPlotTemp['age_group']==ageOver40]     
       
    for idxcallNM, callTypeNM in enumerate(calltypeColsUnderscore):
        dfSubCatCounts=dfPlotTemp.copy()
        
        dfSubCatCounts=dfSubCatCounts[dfSubCatCounts['call_type_cat']==callTypeNM]
        
        sumColumns=['count_calls','cases','first_dose', 'second_dose', 'both_dose', 'first_dose_only']      
        dfSubCatCountsSum=dfSubCatCounts.groupby(
                [weeksColNm,groupbyNm], sort=False)[sumColumns].sum().reset_index()
                
        dfSubCatCountsSum=dfSubCatCountsSum.reset_index()
        if ageOver40 =='All':
            dfSubCatCounts=dfSubCatCounts[['call_type_cat',
                                     'week_start_date', 'week_of_year', 'week_start_date_year',]]     
            dfSubCatCounts=dfSubCatCounts.drop_duplicates()
            dfSubCatCounts=dfSubCatCounts.reset_index(drop=True)
            dfSubCatCounts[sumColumns]=dfSubCatCountsSum[sumColumns]
    
        """Generate suspected first does only time series"""
        dfSubCatCounts['second_dose_shift_3']=dfSubCatCounts['second_dose'].shift(-3)
        dfSubCatCounts['suspected_single_dose_recovered']=(dfSubCatCounts['first_dose']-
                                                                           dfSubCatCounts['second_dose_shift_3'])
            
        dfSubCatCounts.loc[dfSubCatCounts['week_start_date'  ]<datetime.date(2021, 2, 28),
                            'suspected_single_dose_recovered' ]=np.nan
        
        """Calculate moving average"""
        dfSubCatCounts=dfSubCatCounts.sort_values([groupbyNm,weeksColNm])
        RollingAverageCols=['count_calls', 'cases', 'first_dose', 
                            'second_dose','suspected_single_dose_recovered','second_dose_shift_3']

        
        RA_col='count_calls'
        for RA_col in RollingAverageCols:  
            if RA_col==('count_calls'):
                dfSubCatCounts['moving_average'+RA_col] = dfSubCatCounts[RA_col].transform(lambda x: x.rolling(5, 1,center=True).mean())
                
            elif RA_col==('cases'):
                dfSubCatCounts['moving_average'+RA_col] = dfSubCatCounts[RA_col].transform(lambda x: x.rolling(3, 1,center=True).mean())
                
            else:
                dfSubCatCounts['moving_average'+RA_col] = dfSubCatCounts[RA_col].transform(lambda x: x.rolling(3, 1,center=True).mean())

    
        """From April 1st - Process suspected population that only received 1 dose
        - From MS: This difference is also used to estimated number of single doses administered to individuals
        who had recovered from COVID-19 infections, which was plotted from April 1st, 2021, onwards 
        (April 1st 2021 was shortly after the Israel Ministry of Health approved vaccination for this population[41])"""          
        dfSubCatCounts.loc[dfSubCatCounts['week_start_date']<datetime.date(2021, 4, 1),
                            'moving_averagesuspected_single_dose_recovered' ]=0
        dfSubCatCounts.loc[dfSubCatCounts['week_start_date']<datetime.date(2021, 3, 25),
                            'moving_averagesuspected_single_dose_recovered' ]=np.nan                       
        dfSubCatCounts.loc[pd.isnull(dfSubCatCounts['moving_average'+'second_dose_shift_3' ])==True,
                        'moving_average'+'suspected_single_dose_recovered' ]=np.nan            
        dfSubCatCounts['DateStr']=dfSubCatCounts['week_start_date'].astype(str)
       
        
        
        """Figure A - Full Plots"""
        dfSubCatCountsPlot=dfSubCatCounts.copy()
        dfSubCatCountsPlot=dfSubCatCountsPlot.reset_index()
        countCallType='count_calls'
        
        """Plot"""
        plt.figure(figsize=(8,5),dpi=300)
        xindex='DateStr'
        yindex='moving_average'+countCallType
        x=dfSubCatCountsPlot[xindex]
        y=dfSubCatCountsPlot[yindex]
        
        g=sns.lineplot(x=xindex, y=yindex,
                  ci=None ,
                  markers=False, dashes=False, data=dfSubCatCountsPlot)
        plt.show()
        X_labels = [item.get_text() for item in g.get_xticklabels()]   
        dlabelNew=X_labels
        
        figOne,axFigOne=plt.subplots(figsize=(12,5))

        g=sns.lineplot(x=xindex, y=yindex,
                  ci=None ,
                  markers=False, dashes=False, data=dfSubCatCountsPlot,
                  color='red',label=calltypeColsUnderscoreNamingArray[idxcallNM]+' calls')

        yscaleMetricline=1.6
        axFigOne.set_ylim(0, max(y)*yscaleMetricline)

        figOne.suptitle('Ages: '+AgeGroupNamingArray[idxageOver40], fontsize=16)
        
        ax2Dual=axFigOne.twinx()
        y2Str='moving_average'+'first_dose'
        y2=dfSubCatCountsPlot[y2Str]/50000
        
        g2=ax2Dual.plot(dfSubCatCountsPlot[xindex].values, 
                          dfSubCatCountsPlot[y2Str].values/50000,
                    color='Purple',label='1st vaccine doses\n(50,000 doses)',zorder=1)
                        
        y22Str='moving_average'+'second_dose'
        y22=dfSubCatCountsPlot[y22Str]/50000
        g22=ax2Dual.plot(dfSubCatCountsPlot[xindex].values, 
                        dfSubCatCountsPlot[y22Str].values/50000,
                   color='Blue',label='2nd vaccine doses\n(50,000 doses)',zorder=1)
                
        y23Str='moving_average'+'cases'
        y23=dfSubCatCountsPlot[y23Str]/10000
        g23=ax2Dual.plot(dfSubCatCountsPlot[xindex].values, 
                        dfSubCatCountsPlot[y23Str].values/10000,
                    color='grey',label='COVID-19 cases\n(10,000 cases)')
                
        yscaleMetric=1.25
        ax2Dual.set_ylim(0, (y23.append(y22)).max()*yscaleMetric)
        ax2Dual.set_ylabel("Vaccine doses (50,000 doses)\nand COVID-19 cases (10,000 cases)",fontsize=14)
        ax = plt.gca()
        ax.tick_params(axis = 'both', which = 'major', labelsize = 12)
           
        axFigOne.set_xlabel('Date (week\'s starting date)', fontsize=16,)
        
        
        if calltypeColsUnderscoreNamingArray[idxcallNM]=='Acute coronary\nsyndrome':
            axFigOne.set_ylabel('Number of '+'Acute coronary syndrome'+'\n calls, (n)', fontsize=14)
        else:            
            axFigOne.set_ylabel('Number of '+calltypeColsUnderscoreNamingArray[idxcallNM]+'\n calls, (n)', fontsize=14)
             
        """public health advisories at closest week"""
        
        axFigOne.plot(['2020-03-08', '2020-03-08' ], [-1, max(y)*yscaleMetricline],'--',color='cyan',label='1st Public health advisory\n(3-11-2020 to 5-4-2020)')
        axFigOne.plot(['2020-04-26' , '2020-04-26' ], [-1,max(y)*yscaleMetricline],'--',color='cyan',label='Public health advisory lifted\n(5-4-2020)')

        axFigOne.plot(['2020-07-05' , '2020-07-05' ], [-1, max(y)*yscaleMetricline],'--',color='orange',label='2nd Public health advisory\n(7-6-2020 to 10-18-2020)')
        axFigOne.plot(['2020-10-11' , '2020-10-11' ], [-1,max(y)*yscaleMetricline],'--',color='orange',label='Public health advisory lifted\n(10-18-2020)')
        
        axFigOne.plot(['2020-12-20' , '2020-12-20' ], [-1, max(y)*yscaleMetricline],'--',color='pink',label='3rd Public health advisory\n(12-24-2020 to 3-7-2020)')
        axFigOne.plot(['2021-03-07' , '2021-03-07' ], [-1, max(y)*yscaleMetricline],'--',color='pink',label='Public health advisory lifted\n(3-7-2021)')

        """ensure proper labels"""
        handles,labels = axFigOne.get_legend_handles_labels()
        lines2Dual, labels2Dual = ax2Dual.get_legend_handles_labels()
        
        handles =[handles[0]]+lines2Dual+[handles[1],handles[3],handles[5]]
        labels = [labels[0]]+labels2Dual+[labels[1],labels[3],labels[5]]

        g.axes.legend(handles,labels,frameon=False,prop={'size': 10}, loc='upper left',ncol=2)
   
        plt.setp(axFigOne.get_xticklabels(), rotation=90, horizontalalignment='center')
        
        for ind, label in enumerate(axFigOne.get_xticklabels()):
            if ind % 8== 0:  #every 8th label is kept
                label.set_visible(True)
            else:
                label.set_visible(False)
            if ind ==  x.shape[0]-1:
                label.set_visible(True) #keep last label to end of time period
                
        axFigOne.set_zorder(ax2Dual.get_zorder()+1000)
        axFigOne.set_frame_on(False)
        
        plt.savefig(os.getcwd()  +OUTPUT_FOLDER+ "\\FigureA_2023"+
                    calltypeColsUnderscoreNamingArray[idxcallNM].replace('\n','')+ageOver40+".png",bbox_inches='tight',dpi=300)
        
        plt.show()

        
        
        """Figure B - Zoom in plots"""

        dfSubCatCountsPlot=dfSubCatCounts.copy()
        dfSubCatCountsPlot=dfSubCatCountsPlot[dfSubCatCountsPlot['week_start_date' ]>= datetime.date(2020, 10, 15)]
        dfSubCatCountsPlot=dfSubCatCountsPlot.reset_index()
        
        """Plot"""
        plt.figure(figsize=(8,5),dpi=300)
        xindex='DateStr'
        yindex='moving_average'+countCallType
        x=dfSubCatCountsPlot[xindex]
        y=dfSubCatCountsPlot[yindex]
        
        g=sns.lineplot(x=xindex, y=yindex,
                  ci=None ,
                  markers=False, dashes=False, data=dfSubCatCountsPlot)
        plt.show()
        X_labels = [item.get_text() for item in g.get_xticklabels()]   
        dlabelNew=X_labels
        
        figOne,axFigOne=plt.subplots(figsize=(9.5,9))

        g=sns.lineplot(x=xindex, y=yindex,
                  ci=None ,
                  markers=False, dashes=False, data=dfSubCatCountsPlot,
                  color='red',label=calltypeColsUnderscoreNamingArray[idxcallNM]+' calls')
        
        dualyscaleline=1.75
        axFigOne.set_ylim(0, max(y)*dualyscaleline)
        figOne.suptitle('Ages: '+AgeGroupNamingArray[idxageOver40], fontsize=16)
        figOne.subplots_adjust(top=0.92)        
        
        ax2Dual=axFigOne.twinx()
        y2Str='moving_average'+'first_dose'
        y2=dfSubCatCountsPlot[y2Str]/50000
        
        g2=ax2Dual.plot(dfSubCatCountsPlot[xindex].values, 
                          dfSubCatCountsPlot[y2Str].values/50000,
                    color='Purple',label='1st vaccine doses\n(50,000 doses)',zorder=1)
        
        y22Str='moving_average'+'second_dose'
        y22=dfSubCatCountsPlot[y22Str]/50000
        g22=ax2Dual.plot(dfSubCatCountsPlot[xindex].values, 
                        dfSubCatCountsPlot[y22Str].values/50000,
                   color='Blue',label='2nd vaccine doses\n(50,000 doses)',zorder=1)
                
        y22shiftStr='moving_average'+'second_dose_shift_3'
        y22shift=dfSubCatCountsPlot[y22shiftStr]/50000
        g22shift=ax2Dual.plot(dfSubCatCountsPlot[xindex].values, 
                        dfSubCatCountsPlot[y22shiftStr].values/50000,
                        color='Blue',label='2nd vaccine doses shifted\nbackwards in time by\n3-weeks (50,000 doses)',zorder=1,
                   linewidth = 1.75,
                   linestyle=':')
    
        y22shiftStr2='moving_average'+'suspected_single_dose_recovered'
        y22shift2=dfSubCatCountsPlot[y22shiftStr2]/2500
        g22shift2=ax2Dual.plot(dfSubCatCountsPlot[xindex].values, 
                        dfSubCatCountsPlot[y22shiftStr2].values/2500,
                    color='lime',
                    label='Suspected single vaccine doses\nfor recovered individuals\nafter 4-1-2021 (2,500 doses)',
                    zorder=1,
                    linewidth = 1.75,)
        
        y23Str='moving_average'+'cases'
        y23=dfSubCatCountsPlot[y23Str]/10000
        g23=ax2Dual.plot(dfSubCatCountsPlot[xindex].values, 
                        dfSubCatCountsPlot[y23Str].values/10000,
                    color='grey',label='COVID-19 cases\n(10,000 cases)')
        
        dualyscale=1.45
        ax2Dual.set_ylim(0, (y23.append(y22)).max()*dualyscale)
        ax2Dual.set_ylabel("Vaccine doses (50,000 doses)\nand COVID-19 cases (10,000 cases)",fontsize=14)
        ax = plt.gca()
        ax.tick_params(axis = 'both', which = 'major', labelsize = 12)
                   
        axFigOne.set_xlabel('Date (week\'s starting date)', fontsize=16,)

        if calltypeColsUnderscoreNamingArray[idxcallNM]=='Acute coronary\nsyndrome':
            axFigOne.set_ylabel('Number of '+'Acute coronary syndrome'+'\n calls, (n)', fontsize=14)
        else:            
            axFigOne.set_ylabel('Number of '+calltypeColsUnderscoreNamingArray[idxcallNM]+'\n calls, (n)', fontsize=14)    
      
        """public health advisories at closest week"""
        
        axFigOne.plot(['2020-12-20' , '2020-12-20' ], [-1, max(y)*dualyscaleline],'--',color='pink',label='3rd Public health advisory\n(12-24-2020 to 3-7-2020)') 
        axFigOne.plot(['2021-03-07' , '2021-03-07' ], [-1, max(y)*dualyscaleline],'--',color='pink',label='Public health advisory lifted\n(3-7-2021)')
        axFigOne.plot(['2020-12-13', '2020-12-13' ], [-1, max(y)*dualyscaleline],'--',color='darkgreen',label='Vaccinations first began\n(2020-12-19)')

        """ensure proper labels"""
        handles,labels = axFigOne.get_legend_handles_labels()
        lines2Dual, labels2Dual = ax2Dual.get_legend_handles_labels()
        
        handles =[handles[0]]+lines2Dual+[handles[1],handles[3]]
        labels = [labels[0]]+labels2Dual+[labels[1],labels[3]]
        
        g.axes.legend(handles,labels,frameon=True,prop={'size': 10}, loc='upper center',ncol=2)#
        
        plt.setp(axFigOne.get_xticklabels(), rotation=90, horizontalalignment='center')
        
        for ind, label in enumerate(axFigOne.get_xticklabels()):
            if ind % 5== 0:  #every 5th label is kept
                label.set_visible(True)
            else:
                label.set_visible(False)
            if ind ==  x.shape[0]-1: 
                label.set_visible(False) #don't need to keep last week label
                
        axFigOne.set_zorder(ax2Dual.get_zorder()+1000)
        axFigOne.set_frame_on(False)
        
        plt.savefig(os.getcwd()  +OUTPUT_FOLDER+ "\\FigureB_2023"+
                    calltypeColsUnderscoreNamingArray[idxcallNM].replace('\n','')+ageOver40+".png",bbox_inches='tight',dpi=300)
        
        plt.show()

        




