import scipy.stats as stats
import numpy as np
from pingouin import power_corr

"""validation approach 1"""
n = 128    # sample size
alpha = 0.05  # significance level
"""
The weekly CA call count with the cumulative bi-weekly
(current and prior week) 1st and 2nd doses count time-series (0.209, P < 0.05).
"""
r = 0.209 
print(power_corr(r=r,n=n,alpha=alpha))

"""
The sum of the weekly CA and ACS call counts with the cumulative bi-weekly 
(current and prior week) 1st and 2nd doses count time-series (0.457, P < 0.001).
"""
r = 0.457
print(power_corr(r=r,n=n,alpha=alpha))

"""
The CA weekly call count with the cumulative three-week (current and two prior
weeks) new COVID-19 infections count time-series (0.076, P = 0.392).
"""
r = 0.076
print(power_corr(r=r,n=n,alpha=alpha))

"""
The sum of CA and ACS weekly call counts with the cumulative three-week 
(current and two prior weeks) new COVID-19 infections 
count time-series (0. 410, P = 0.001).
"""
r = 0.410
print(power_corr(r=r,n=n,alpha=alpha))




"""validation approach 2"""


def post_hoc_power(r,n,alpha):
    # test statistic - distributed approximately as Student's t-distribution with n − 2 degrees of freedom under the null hypothesis https://archive.org/details/advancedtheoryof0001kend
    t = r * np.sqrt((n - 2) / (1 - r**2))

    df = n - 2  # degrees of freedom
    tcrit = abs(stats.t.ppf(alpha/2, df)) 

    # power
    ncp = np.abs(t)  # non-centrality parameter
    power = 1 - stats.nct.cdf(tcrit, df, ncp) 
    print('Power:', power)
    return power    


n = 128    # sample size
alpha = 0.05  # significance level


"""
The weekly CA call count with the cumulative bi-weekly
(current and prior week) 1st and 2nd doses count time-series (0.209, P < 0.05).
"""
r = 0.209 
post_hoc_power(r,n,alpha)

"""
The sum of the weekly CA and ACS call counts with the cumulative bi-weekly 
(current and prior week) 1st and 2nd doses count time-series (0.457, P < 0.001).
"""
r = 0.457
post_hoc_power(r,n,alpha)

"""
The CA weekly call count with the cumulative three-week (current and two prior
weeks) new COVID-19 infections count time-series (0.076, P = 0.392).
"""
r = 0.076
post_hoc_power(r,n,alpha)

"""
The sum of CA and ACS weekly call counts with the cumulative three-week 
(current and two prior weeks) new COVID-19 infections 
count time-series (0. 410, P = 0.001).
"""
r = 0.410
post_hoc_power(r,n,alpha)

