library('RcmdrMisc')
library(MASS)
library(rsq)
library(broom)



setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

#read weekly counts
data_master <- read.csv("data/weekly_counts.csv", header=TRUE)
data_master<-data_master[!(data_master$fourty_and_above==1),]

CoVarName='under_40_CA_ACS'
outputFolder='output_under_40_CA_ACS_regression/'

#save console output
sink(file=paste(outputFolder,CoVarName,'model_output.txt',sep = ""))

var_list_cases <- c('cases_popnormalized',
                     'cases_2weekspopnormalized',
                     'cases_3weekspopnormalized',
                     'cases_4weekspopnormalized',
                     'cases_5weekspopnormalized',
                     'cases_6weekspopnormalized')

#2 week dose label
var_dose_label<-2
var_list_doses <- c(paste('both_dose_',var_dose_label,'weekspopnormalized',sep=''))

#feature selection model
summary_df <- data.frame()
rowcounter<- 1
for (i in seq_along(var_list_doses)){
  for (k in seq_along(var_list_cases)){
    
    mod <- as.formula(sprintf("count_calls~%s+%s+call_type_ca+any_lockdown+year_2019+year_2020", var_list_doses[i],var_list_cases[k]))
    glmmodel  <- glm.nb(formula = mod, data = data_master,maxit=1000)
    
    step.model<-stepwise(glmmodel, 
                         direction = "backward/forward", 
                         criterion = "BIC",trace=FALSE)
    print(summary(step.model))
    
    #summary of regression
    summary_df[rowcounter,1] <- k
    summary_df[rowcounter,2] <- var_dose_label
    summary_df[rowcounter,3] <- BIC(step.model)
    summary_df[rowcounter,4]<-rsq(step.model,adj=TRUE)
    summary_df[rowcounter,5]<-step.model$coefficients[var_list_cases[k]]
    
    if (is.na(step.model$coefficients[var_list_cases[k]])==FALSE){
      summary_df[rowcounter,6]<-summary(step.model)$coefficients[,"Pr(>|z|)"][[var_list_cases[k]]]
    }else{
      summary_df[rowcounter,6]=NA
    }
    summary_df[rowcounter,7]<-step.model$coefficients[var_list_doses[i]]
    if (is.na(step.model$coefficients[var_list_doses[i]])==FALSE){
      summary_df[rowcounter,8]<-summary(step.model)$coefficients[,"Pr(>|z|)"][[var_list_doses[i]]]
    }else{
      summary_df[rowcounter,8]=NA
    }

    tidy_lmfit <- tidy(step.model)
    
    (est <- cbind(Estimate = coef(step.model), confint(step.model,level = 0.95)))
    coeff_output_df<-exp(est)
    coeff_output_df<-as.data.frame(coeff_output_df)
    coeff_output_df$pvalues<-tidy_lmfit$p.value
    
    #save individual model
    write.csv(coeff_output_df, paste(outputFolder,CoVarName,'_exp_feat_sel',var_dose_label,k,'.csv',sep = ""))
    rowcounter<- 1+rowcounter
    
  }
}
colnames(summary_df) <- c('Cases_Weeks_Sumed','Doses_Weeks_Sumed','BIC', 'Adjusted_R^2','Cases_Coeff','Cases_P','Doses_Coeff','Doses_P')
write.csv(summary_df, paste(outputFolder,CoVarName,'_feat_sel_summary.csv',sep = ""))




#no feature selection model
summary_df <- data.frame()
rowcounter<- 1
for (i in seq_along(var_list_doses)){
  for (k in seq_along(var_list_cases)){
    
    mod <- as.formula(sprintf("count_calls~%s+%s+call_type_ca+any_lockdown+year_2019+year_2020", var_list_doses[i],var_list_cases[k]))
    
    full_model  <- glm.nb(formula = mod, data = data_master,maxit=1000)
    print(summary(full_model))
    
    #summary of regression
    summary_df[rowcounter,1] <- k
    summary_df[rowcounter,2] <- var_dose_label
    summary_df[rowcounter,3] <- BIC(full_model)
    summary_df[rowcounter,4]<-rsq(full_model,adj=TRUE)
    summary_df[rowcounter,5]<-full_model$coefficients[var_list_cases[k]]
    if (is.na(full_model$coefficients[var_list_cases[k]])==FALSE){
      summary_df[rowcounter,6]<-summary(full_model)$coefficients[,"Pr(>|z|)"][[var_list_cases[k]]]
    }else{
      summary_df[rowcounter,6]=NA
    }
    summary_df[rowcounter,7]<-full_model$coefficients[var_list_doses[i]]
    if (is.na(full_model$coefficients[var_list_doses[i]])==FALSE){
      summary_df[rowcounter,8]<-summary(full_model)$coefficients[,"Pr(>|z|)"][[var_list_doses[i]]]
    }else{
      summary_df[rowcounter,8]=NA
    }

    tidy_lmfit <- tidy(full_model)
    
    (est <- cbind(Estimate = coef(full_model), confint(full_model,level = 0.95)))
    coeff_output_df<-exp(est)
    coeff_output_df<-as.data.frame(coeff_output_df)
    coeff_output_df$pvalues<-tidy_lmfit$p.value
    
    #save individual model
    write.csv(coeff_output_df, paste(outputFolder,CoVarName,'_exp_full',var_dose_label,k,'.csv',sep = ""))
    
    rowcounter<- 1+rowcounter
    
  }
}
colnames(summary_df) <- c('Cases_Weeks_Sumed','Doses_Weeks_Sumed','BIC', 'Adjusted_R^2','Cases_Coeff','Cases_P','Doses_Coeff','Doses_P')
write.csv(summary_df, paste(outputFolder,CoVarName,'_full_summary.csv',sep = ""))

#export console output
sink()
