import numpy as np
import os
import pandas as pd
import statsmodels.stats.proportion as ssp

OUTPUT_FOLDER='\\output_tables' 
DATA_SOURCE='\\data'

"""import CA table counts"""
table_ca_counts=pd.read_csv(os.getcwd() + DATA_SOURCE +"/supplemental_table_ca_counts.csv").drop(['Unnamed: 0'],axis=1)

"""calculate death and resuscitation rates for CA"""
table_ca_counts['death_proportion']=table_ca_counts['death']/table_ca_counts['counts']
table_ca_counts['resuscitation_proportion']=table_ca_counts['resuscitation']/table_ca_counts['counts']

table_ca_counts['death_shift']=table_ca_counts['death'].shift()
table_ca_counts['resuscitation_shift']=table_ca_counts['resuscitation'].shift()
table_ca_counts['counts_shift']=table_ca_counts['counts'].shift()
table_ca_counts['death_proportion_p_value']=np.nan
table_ca_counts['resuscitation_proportion_p_value']=np.nan

for index_df, row_df in table_ca_counts.iterrows():
    counts=row_df['counts']
    resuscitation=row_df['resuscitation']
    death=row_df['death']
    counts_shift=row_df['counts_shift']
    resuscitation_shift=row_df['resuscitation_shift']
    death_shift=row_df['death_shift']
    labels=row_df['labels']
    if ('2019' in labels):
        table_ca_counts.loc[index_df,'death_proportion_p_value']='-'
        table_ca_counts.loc[index_df,'resuscitation_proportion_p_value']='-'
    else:         
        data_death = pd.DataFrame({
            "Event": [death, death_shift],
            "Total_Event": [counts, counts_shift]
            })
        (chi2_death, p_death, arr_death) = ssp.proportions_chisquare(count = data_death.Event, nobs = data_death.Total_Event)
        table_ca_counts.loc[index_df,'death_proportion_p_value']=p_death
        data_resus= pd.DataFrame({
            "Event": [resuscitation, resuscitation_shift],
            "Total_Event": [counts, counts_shift]
            })
        (chi2_resus, p_resus, arr_resus) = ssp.proportions_chisquare(count = data_resus.Event, nobs = data_resus.Total_Event)
        table_ca_counts.loc[index_df,'resuscitation_proportion_p_value']=p_resus

table_ca_counts=table_ca_counts[['labels', 'counts', 'death', 'resuscitation', 'death_proportion',
       'resuscitation_proportion', 'death_proportion_p_value',
       'resuscitation_proportion_p_value']]


table_ca_counts.to_csv(os.getcwd() + OUTPUT_FOLDER + "\\supplemental_table_ca_counts_p_values.csv",encoding='utf-8-sig')   









